def f1(x: int = ...) -> None: ...
def f2(x: int = 3) -> None: ...  # Y011 Default values for typed arguments must be "..."
def f3(*, x: int = ...) -> None: ...
def f4(*, x: int = 3) -> None: ...  # Y011 Default values for typed arguments must be "..."
def f5(x=3) -> None: ...  # Y014 Default values for arguments must be "..."
def f6(*, x: int) -> None: ...
def f7(x, y: int = 3) -> None: ...  # Y011 Default values for typed arguments must be "..."
def f8(x, y: int = ...) -> None: ...
